//////////////////////////////////////
// author : Dirk Duschinger         //
// email  : dirk.duschigner@cern.ch //
//////////////////////////////////////

#include "EpsParser.h"

EpsParser::EpsParser()
{
};

EpsParser::EpsParser(const char* fName)
{
  ReadFile(fName);
  ReadEpsResolution();
  GetListOfCommands();
  GetDictOfDict();
  ReplaceDictCommandsWithEpsCommands();
  GetListOfObjects();
  std::vector<EpsPolygon*>::iterator it = ListOfObjects.begin();
  for (; it!=ListOfObjects.end(); ++it);
};

EpsParser::~EpsParser()
{
};

void EpsParser::ReadFile(const char* fName)
{
  //f = new TFile(fName);
  std::ifstream input(fName);
  bool flag_setup = true;
  for( std::string line; getline( input, line ); )
  {
    if (flag_setup)
    {
      if (line == "%%EndSetup")
        flag_setup = false;
      FStrings.push_back(line);
    }
    else
    {
      ObjStrings.push_back(line);
    }
  }
};

void EpsParser::ReadEpsResolution()
{
  std::list<std::string>::iterator it = FStrings.begin();
  for(; it!= FStrings.end(); ++it)
  {
    if ((*it).find("BoundingBox") != std::string::npos)
    {
      std::vector<std::string> vSplit = split(*it,' ',true);
      epsXres = atoi(vSplit.at(3).c_str());
      epsYres = atoi(vSplit.at(4).c_str());
    }
  }
}

void EpsParser::GetListOfCommands()
{
  std::list<std::string>::iterator it = ObjStrings.begin();
  for (; it!=ObjStrings.end(); ++it)
  {
    std::vector<std::string> vSplit = split(*it,' ',true);
    ListOfCommands.insert(ListOfCommands.end(), vSplit.begin(), vSplit.end());
  }
}

void EpsParser::GetDictOfDict()
{
  std::list<std::string>::iterator it = FStrings.begin();

  for(; it!=FStrings.end(); ++it)
  {
    if ((*it).find("def") == std::string::npos)
      continue;
    int flag = 0;
    int nBraces = 0;
    std::string dict_name = "";
    std::string dict_entry = "";

    for(size_t i = 0; i!=it->length(); i++)
    {
      if (flag == 0)
      {
        if (it->at(i) == '/')
        {
          flag = 1; //flag found
          continue;
        }
      }
      if (flag == 1)
      {
        if (it->at(i) == '{')
        {
          flag = 2;
          nBraces++;
          continue;
        }
        else
        {
          if (it->at(i)== ' ')
            continue;
          dict_name+=it->at(i);
        }
      }
      if (flag == 2)
      {
        if (it->at(i) == '{')
          nBraces++;
        if (it->at(i) == '}')
          nBraces--;
        if (nBraces == 0)
        {
          flag = 0;
          // remove leading and trailing spaces
          dict_entry = dict_entry.substr(dict_entry.find_first_not_of(' '), dict_entry.find_last_not_of(' ')+1);
          DictOfDict.insert(std::pair<std::string,std::string>(dict_name,dict_entry));
          dict_name = "";
          dict_entry = "";
          continue;
        }
        dict_entry+=it->at(i);
      }
    }
  }
}

void EpsParser::ReplaceDictCommandsWithEpsCommands()
{
  bool flag_rerun = true;
  while(flag_rerun)
  {
    std::vector<std::string> tmpListOfCommands;
    flag_rerun = false;
    std::vector<std::string>::iterator it = ListOfCommands.begin();
    for(; it != ListOfCommands.end(); ++it)
    {
      std::map<std::string,std::string>::iterator mapIt= DictOfDict.find(*it);
      if (mapIt != DictOfDict.end())
      {
        flag_rerun = true;
        std::vector<std::string> vSplit = split(mapIt->second,' ',true);
        tmpListOfCommands.insert(tmpListOfCommands.end(), vSplit.begin(), vSplit.end());
      }
      else
      {
        tmpListOfCommands.push_back(*it);
      }
    }
    ListOfCommands = tmpListOfCommands;
  }
}

void EpsParser::GetListOfObjects()
{
  bool flag_str = false;
  std::vector<std::string>::iterator it = ListOfCommands.begin();
  std::string tmpString = "";
  for(; it!=ListOfCommands.end(); ++it)
  {
    char* pEnd;
    double d = std::strtod(it->c_str(),&pEnd); //pEnd denotes the part of the string which could not be converted to double
    if (*pEnd) //if string is not a pure double, than *pEnd is changed and thus different from NULL
    {
      if ((*it).find("(")!=std::string::npos or flag_str)
      {
        flag_str = true;
        if ((*it).find(")")!=std::string::npos)
        {
          tmpString += *it;
          // remove braces
          tmpString.pop_back();
          tmpString.erase(0,1);
          sStack.push_back(tmpString);
          flag_str = false;
          tmpString = "";
        }
        else
        {
          tmpString += *it;
        }
      }
      EpsCommands epsc = EpsCommands(*it, &ListOfObjects, &sStack, &dStack);
    }
    else
      dStack.push_back(d);

  }
}


/*
    def __GetListOfObjects__(self):
        for ind, c in enumerate(self.__ListOfCommands__):
                if "(" in c or flag_str == True:
                    #TODO: care about setting braces
                    c = c.replace("\)","")
                    c = c.replace("\(","")
                    if ")" in c:
                        self.__Stack__.append(c)
                        flag_str = False
                    else:
                        self.__Stack__.append(c)
                        flag_str = True
                    continue
                if "" == c:
                    continue
                self.__CurrentListItem__ = ind
                EpsCommands(self,c)
                #print "routine",self.__ListOfObjects__
*/

void EpsParser::Draw(WINDOW* w)
{
  Draw(w, epsXres*4., epsYres*4.);
}

void EpsParser::Draw(WINDOW* w, int eps_xres, int eps_yres)
{
  std::vector<EpsPolygon*>::iterator it = ListOfObjects.begin();
  for (; it!=ListOfObjects.end(); ++it)
  {
    (*it)->Draw(w, eps_xres,eps_yres);
    doupdate();
  }
}

std::vector<std::string> &split(const std::string &s, char delim, std::vector<std::string> &elems, bool noEmpty = false)
{
  std::stringstream ss(s);
  std::string item;
  while (std::getline(ss, item, delim))
  {
    if ((noEmpty) && item == "")
    {
      continue;
    }
    elems.push_back(item);
  }
  return elems;
}

std::vector<std::string> split(const std::string &s, char delim, bool noEmpty = false)
{
  std::vector<std::string> elems;
  split(s, delim, elems, noEmpty);
  return elems;
}
